// {{{RME classifier 'Logical View::RQARTClasses::CPP::V6_4_100_0::RQARTAbstractTestHarness'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "RQARTAbstractTestHarness.h"
#endif

#include <RTSystem/VVIG.h>
#include <RQARTAbstractTestHarness.h>
#include <RQARTIncarnateArguments.h>
#include <RQARTStartTestNData.h>
extern const RTActorClass RQARTTCPServer;

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static RTActor * new_RQARTAbstractTestHarness_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new RQARTAbstractTestHarness_Actor( _rts, _ref );
}

const RTActorClass RQARTAbstractTestHarness =
{
	&RQARTAbstractTestBase
  , "RQARTAbstractTestHarness"
  , (RTVersionId)0
  , 0
  , (const RTRelayDescriptor *)0
  , new_RQARTAbstractTestHarness_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Ready"
  , "WaitingForCleanUp"
  , "RunningTest"
  , "Failed"
  , "AreAllDriversCompleted"
};

static const RTInterfaceDescriptor rtg_interfaces_RQART_TCPServer[] =
{
	{
		"AddIn"
	  , 1
	}
};

static const RTBindingDescriptor rtg_bindings_RQART_TCPServer[] =
{
	{
		0
	  , &RQARTAddInProtocol::Base::rt_class
	}
};

const RTTypeModifier rtg_tm_RQARTAbstractTestHarness_Actor_nNumDrivers =
{
	RTNumberConstant
  , 1
  , 1
};

const RTTypeModifier rtg_tm_RQARTAbstractTestHarness_Actor_nMinRunTimes =
{
	RTNumberConstant
  , 1
  , 1
};

#define SUPER RQARTAbstractTestBase_Actor

RQARTAbstractTestHarness_Actor::RQARTAbstractTestHarness_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RQARTAbstractTestBase_Actor( rtg_rts, rtg_ref )
	, nCompletedDrivers( 0 )
	, nNumDrivers( 0 )
	, nMaxTime( -1 )
	, nTimerCount( 0 )
	, nMinRunTimes( 0 )
	, nMaxInstances( 1 )
	, nCurrentInstances( 1 )
	, nEnvironmentIndex( 0 )
	, nRQARTControllerVersion( 1 )
{
}

RQARTAbstractTestHarness_Actor::~RQARTAbstractTestHarness_Actor( void )
{
	DestructorHelper();
}

// {{{RME operation 'StartTimeoutTimer()'
void RQARTAbstractTestHarness_Actor::StartTimeoutTimer( void )
{
	// {{{USR
	// if we have a maximum time specified, then start a timer to enforce it
	if (nMaxTime > 0)
	{
		const int zero = 0;
		timeoutTid = RQART_Timer.informIn(RTTimespec(nMaxTime, 0), (const void *)&zero,
			&RTType_int);
		if (!timeoutTid.isValid())
		{
			RQART_Log.log("Timeout timer could not be started.");
			RQARTExceptionCode exCode = eRQARTTimerCreateError;
			RQART_Exception.error(RTTypedValue(&exCode)).raise();
			return;
		}
	}
	// }}}USR
}
// }}}RME

// {{{RME operation 'CancelTimeoutTimer()'
void RQARTAbstractTestHarness_Actor::CancelTimeoutTimer( void )
{
	// {{{USR
	// cancel the outstanding timer
	if (timeoutTid.isValid())
	{
		RQART_Timer.cancelTimer(timeoutTid);
	}
	// }}}USR
}
// }}}RME

// {{{RME operation 'PrepareToStartTest(const int *const)'
void RQARTAbstractTestHarness_Actor::PrepareToStartTest( const int * const minRunTimes )
{
	// {{{USR
	// general test setup

	nCompletedDrivers = 0;

	// set the min run times.
	int i;
	for (i = 0; i < nCurrentInstances; i++)
		nMinRunTimes[i] = minRunTimes[i];

	// Start the timers
	StartMinRunTimeTimer(nEnvironmentIndex);
	StartTimeoutTimer();

	// }}}USR
}
// }}}RME

// {{{RME operation 'CompleteTest()'
void RQARTAbstractTestHarness_Actor::CompleteTest( void )
{
	// {{{USR

	// }}}USR
}
// }}}RME

// {{{RME operation 'DestroyDrivers()'
void RQARTAbstractTestHarness_Actor::DestroyDrivers( void )
{
	// {{{USR

	// }}}USR
}
// }}}RME

// {{{RME operation 'DestroyCUTs()'
void RQARTAbstractTestHarness_Actor::DestroyCUTs( void )
{
	// {{{USR

	// }}}USR
}
// }}}RME

// {{{RME operation 'Reset()'
void RQARTAbstractTestHarness_Actor::Reset( void )
{
	// {{{USR
	// reset the test by destroying all capsules.
	DestroyDrivers();
	DestroyCUTs();
	if (nRQARTControllerVersion > 1)
	{
		// Notify the controller that the reset is done.
		addIn.ResetCompleted().send();
	}
	// }}}USR
}
// }}}RME

// {{{RME operation 'CreateCapsuleRoleIfNecessary(RTActorRef &,const RQARTIncarnateArguments *)'
bool RQARTAbstractTestHarness_Actor::CreateCapsuleRoleIfNecessary( RTActorRef & capsuleRole, const RQARTIncarnateArguments * args )
{
	// {{{USR
	// incarnate the specified role based on the incarnation arguments only if
	// it hasn't already been created. This method is used to create capsules
	// that might have already been created in a previous test.

	int index;
	RTActorId capsuleId;

	// Determine the index of the role
	if (capsuleRole.size() == 1)
	{
		// role is not replicated
		if(!args || args->cardinalityIndex < 0)
			index = 0;
		else
			index = args->cardinalityIndex;
	}
	else
	{
		// role is replicated
		if(!args || args->cardinalityIndex < 0)
		{
			// Don't know the index, create the role now.
			return CreateCapsuleRole(capsuleRole, args);
		}
		else
			index = args->cardinalityIndex;
	}

	// See if the role has been created already
	capsuleId = RQART_Frame.incarnationAt(capsuleRole, index);
	if(!capsuleId.isValid())
		return CreateCapsuleRole(capsuleRole, args); // create it
	else
		return true; // already created

	// }}}USR
}
// }}}RME

// {{{RME operation 'StartMinRunTimeTimer(int)'
void RQARTAbstractTestHarness_Actor::StartMinRunTimeTimer( int nInstanceIndex )
{
	// {{{USR
	// if we have a minimum time specified, then start a timer to enforce it
	if (nMinRunTimes[nInstanceIndex] > 0)
	{
		RTTimerId minRunTimeTid = RQART_Timer.informIn(
			RTTimespec(nMinRunTimes[nInstanceIndex], 0),
			(const void *)&nCurrentTest, &RTType_int);
		if (minRunTimeTid.isValid())
			nTimerCount++;
		else
		{
			RQART_Log.log("Minimum run time timer could not be started.");
			RQARTExceptionCode exCode = eRQARTTimerCreateError;
			RQART_Exception.error(RTTypedValue(&exCode)).raise();
			return;
		}
	}
	// }}}USR
}
// }}}RME

// {{{RME operation 'CleanUpTest()'
void RQARTAbstractTestHarness_Actor::CleanUpTest( void )
{
	// {{{USR

	// }}}USR
}
// }}}RME

// {{{RME operation 'InitializeTest(const int *const)'
void RQARTAbstractTestHarness_Actor::InitializeTest( const int * const minRunTimes )
{
	// {{{USR

	// }}}USR
}
// }}}RME

// {{{RME operation 'ZeroMinRunTimes()'
void RQARTAbstractTestHarness_Actor::ZeroMinRunTimes( void )
{
	// {{{USR
	int i;
	for (i = 0; i < nMaxInstances; i++)
		nMinRunTimes[i] = 0; 
	// }}}USR
}
// }}}RME

// {{{RME operation 'DestructorHelper()'
void RQARTAbstractTestHarness_Actor::DestructorHelper( void )
{
	// {{{USR
	if (nMinRunTimes)
		delete [] nMinRunTimes;
	if (nNumDrivers)
		delete [] nNumDrivers;
	// }}}USR
}
// }}}RME

int RQARTAbstractTestHarness_Actor::_followOutV( RTBindingEnd & rtg_end, int rtg_compId, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_compId )
	{
	case 1:
		// RQART_TCPServer
		switch( rtg_portId )
		{
		case 0:
			// AddIn
			if( rtg_repIndex < 1 )
			{
				// addIn
				rtg_end.port = &addIn;
				rtg_end.index = rtg_repIndex;
				return 1;
			}
			break;
		default:
			break;
		}
	default:
		break;
	}
	return RQARTAbstractTestBase_Actor::_followOutV( rtg_end, rtg_compId, rtg_portId, rtg_repIndex );
}

// {{{RME enter ':TOP:Failed'
#define CALLSUPER RQARTAbstractTestBase_Actor::enter5_Failed()
INLINE_METHODS void RQARTAbstractTestHarness_Actor::enter5_Failed( void )
{
	// {{{USR
	// clean up
	DestroyDrivers();
	DestroyCUTs();

	// }}}USR
}
#undef CALLSUPER
// }}}RME

void RQARTAbstractTestHarness_Actor::enterStateV( void )
{
	switch( getCurrentState() )
	{
	case 5:
		enter5_Failed();
		break;
	default:
		RQARTAbstractTestBase_Actor::enterStateV();
		break;
	}
}

// {{{RME transition ':TOP:Junction1:timeout' guard '1'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::guard2_timeout_event1
#define CALLSUPER RQARTAbstractTestBase_Actor::guard2_timeout_event1( rtdata, rtport )
INLINE_METHODS int RQARTAbstractTestHarness_Actor::guard2_timeout_event1( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	return *(int *)rtdata == 0;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Junction1:timeout'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition2_timeout
#define CALLSUPER RQARTAbstractTestBase_Actor::transition2_timeout( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition2_timeout( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	// This test timed out. Shut it down.
	addIn.TestTimedOut().send(Panic);
	// clean up
	Reset();

	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Ready:Junction2:Reset'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition3_Reset
#define CALLSUPER RQARTAbstractTestBase_Actor::transition3_Reset( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition3_Reset( const void * rtdata, RQARTAddInProtocol::Base * rtport )
{
	// {{{USR
	Reset();
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Ready:Junction5:SetMaxTime'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition4_SetMaxTime
#define CALLSUPER RQARTAbstractTestBase_Actor::transition4_SetMaxTime( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition4_SetMaxTime( const int * rtdata, RQARTAddInProtocol::Base * rtport )
{
	// {{{USR
	nMaxTime = *rtdata;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:RunningTest:Junction4:timerCompleted' guard '1'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::guard5_timerCompleted_event1
#define CALLSUPER RQARTAbstractTestBase_Actor::guard5_timerCompleted_event1( rtdata, rtport )
INLINE_METHODS int RQARTAbstractTestHarness_Actor::guard5_timerCompleted_event1( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	return *(int *)rtdata != 0;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:RunningTest:Junction4:timerCompleted'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition5_timerCompleted
#define CALLSUPER RQARTAbstractTestBase_Actor::transition5_timerCompleted( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition5_timerCompleted( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	nTimerCount--;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Junction2:timerCompleted' guard '1'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::guard6_timerCompleted_event1
#define CALLSUPER RQARTAbstractTestBase_Actor::guard6_timerCompleted_event1( rtdata, rtport )
INLINE_METHODS int RQARTAbstractTestHarness_Actor::guard6_timerCompleted_event1( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	return *(int *)rtdata != 0;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Junction2:timerCompleted'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition6_timerCompleted
#define CALLSUPER RQARTAbstractTestBase_Actor::transition6_timerCompleted( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition6_timerCompleted( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	nTimerCount--;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:WaitingForCleanUp:Junction2:cleanUp'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition7_cleanUp
#define CALLSUPER RQARTAbstractTestBase_Actor::transition7_cleanUp( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition7_cleanUp( const void * rtdata, RQARTAddInProtocol::Base * rtport )
{
	// {{{USR
	// Clean up the test
	CleanUpTest();
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Ready:Junction8:StartTest'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition8_StartTest
#define CALLSUPER RQARTAbstractTestBase_Actor::transition8_StartTest( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition8_StartTest( const void * rtdata, RQARTAddInProtocol::Base * rtport )
{
	// {{{USR
	// setup this test and start it
	nCurrentTest = *((int *)rtdata);
	InitializeTest(((int*)rtdata) + 1);
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:RunningTest:Junction1:DriverCompleted'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition9_DriverCompleted
#define CALLSUPER RQARTAbstractTestBase_Actor::transition9_DriverCompleted( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition9_DriverCompleted( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	nCompletedDrivers++;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:AreAllDriversCompleted:True'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition11_True
#define CALLSUPER RQARTAbstractTestBase_Actor::transition11_True( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition11_True( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	// this test has completed
	CompleteTest();

	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Junction4:errorException'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition12_errorException
#define CALLSUPER RQARTAbstractTestBase_Actor::transition12_errorException( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition12_errorException( const void * rtdata, Exception::Base * rtport )
{
	// {{{USR
	RQARTExceptionCode exCode = *(RQARTExceptionCode *)rtdata;

	switch (exCode)
	{
	default:
		// notify the addin
		addIn.TestFailed(exCode).send(Panic);
		break;
	}

	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Junction5:DriverFailed'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition13_DriverFailed
#define CALLSUPER RQARTAbstractTestBase_Actor::transition13_DriverFailed( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition13_DriverFailed( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	RQARTExceptionCode exCode = *(RQARTExceptionCode *)rtdata;

	switch (exCode)
	{
	default:
		// notify the addin
		addIn.TestFailed(exCode).send(Panic);
		break;
	}
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Ready:Junction9:SetControllerVersion'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition14_SetControllerVersion
#define CALLSUPER RQARTAbstractTestBase_Actor::transition14_SetControllerVersion( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition14_SetControllerVersion( const int * rtdata, RQARTAddInProtocol::Base * rtport )
{
	// {{{USR
	nRQARTControllerVersion = *rtdata;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Failed:Junction7:ignoreMessages'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition15_ignoreMessages
#define CALLSUPER RQARTAbstractTestBase_Actor::transition15_ignoreMessages( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestHarness_Actor::transition15_ignoreMessages( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	// We're already in the test failed state and have executed the entry action
	// to clean up. Ignore any additional messages.
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain1_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain12_errorException( void )
{
	// transition ':TOP:Junction4:errorException'
	rtgChainBegin( 1, "errorException" );
	exitToChainState( 1, rtg_parent_state );
	rtgTransitionBegin();
	transition12_errorException( msg->data, (Exception::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 5 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain2_timeout( void )
{
	// transition ':TOP:Junction1:timeout'
	rtgChainBegin( 1, "timeout" );
	exitToChainState( 1, rtg_parent_state );
	rtgTransitionBegin();
	transition2_timeout( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain6_timerCompleted( void )
{
	// transition ':TOP:Junction2:timerCompleted'
	rtgChainBegin( 1, "timerCompleted" );
	rtgTransitionBegin();
	transition6_timerCompleted( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain8_StartTest( void )
{
	// transition ':TOP:Ready:Junction8:StartTest'
	rtgChainBegin( 2, "StartTest" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition8_StartTest( msg->data, (RQARTAddInProtocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain3_Reset( void )
{
	// transition ':TOP:Ready:Junction2:Reset'
	rtgChainBegin( 2, "Reset" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition3_Reset( msg->data, (RQARTAddInProtocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain4_SetMaxTime( void )
{
	// transition ':TOP:Ready:Junction5:SetMaxTime'
	rtgChainBegin( 2, "SetMaxTime" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition4_SetMaxTime( (const int *)msg->data, (RQARTAddInProtocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain14_SetControllerVersion( void )
{
	// transition ':TOP:Ready:Junction9:SetControllerVersion'
	rtgChainBegin( 2, "SetControllerVersion" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition14_SetControllerVersion( (const int *)msg->data, (RQARTAddInProtocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain7_cleanUp( void )
{
	// transition ':TOP:WaitingForCleanUp:Junction2:cleanUp'
	rtgChainBegin( 3, "cleanUp" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition7_cleanUp( msg->data, (RQARTAddInProtocol::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain5_timerCompleted( void )
{
	// transition ':TOP:RunningTest:Junction4:timerCompleted'
	rtgChainBegin( 4, "timerCompleted" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_timerCompleted( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	if( choicePoint1_AreAllDriversCompleted( msg->data, (Timing::Base *)msg->sap() ) )
		chain11_True();
	else
		chain10_False();
}

// {{{RME choicePoint ':TOP:AreAllDriversCompleted'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::choicePoint1_AreAllDriversCompleted
#define CALLSUPER RQARTAbstractTestBase_Actor::choicePoint1_AreAllDriversCompleted( rtdata, rtport )
INLINE_METHODS int RQARTAbstractTestHarness_Actor::choicePoint1_AreAllDriversCompleted( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	return (nCompletedDrivers >= nNumDrivers[nCurrentTest - 1]) &&
		(nTimerCount == 0);
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain11_True( void )
{
	// transition ':TOP:AreAllDriversCompleted:True'
	rtgChainBegin( 6, "True" );
	rtgTransitionBegin();
	transition11_True( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain10_False( void )
{
	// transition ':TOP:AreAllDriversCompleted:False'
	rtgChainBegin( 6, "False" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 4 );
}

INLINE_CHAINS void RQARTAbstractTestHarness_Actor::chain15_ignoreMessages( void )
{
	// transition ':TOP:Failed:Junction7:ignoreMessages'
	rtgChainBegin( 5, "ignoreMessages" );
	rtgTransitionBegin();
	transition15_ignoreMessages( msg->data, msg->sap() );
	rtgTransitionEnd();
}

void RQARTAbstractTestHarness_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain1_Initial();
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::RQARTClasses::CPP::V6_4_100_0::RQARTAbstractTestBase' port 'RQART_Exception'
				switch( signalIndex )
				{
				case Exception::Base::rti_error:
					chain12_errorException();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 4:
				// {{{RME port 'RQART_Timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					if( guard2_timeout_event1( msg->data, (Timing::Base *)msg->sap() ) )
					{
						chain2_timeout();
						return;
					}
					if( guard6_timerCompleted_event1( msg->data, (Timing::Base *)msg->sap() ) )
					{
						chain6_timerCompleted();
						return;
					}
					break;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Ready'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 5:
				// {{{RME port 'addIn'
				switch( signalIndex )
				{
				case RQARTAddInProtocol::Base::rti_StartTest:
					chain8_StartTest();
					return;
				case RQARTAddInProtocol::Base::rti_Reset:
					chain3_Reset();
					return;
				case RQARTAddInProtocol::Base::rti_SetMaxTime:
					chain4_SetMaxTime();
					return;
				case RQARTAddInProtocol::Base::rti_SetRQARTControllerVersion:
					chain14_SetControllerVersion();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:WaitingForCleanUp'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 5:
				// {{{RME port 'addIn'
				switch( signalIndex )
				{
				case RQARTAddInProtocol::Base::rti_CleanUpTest:
					chain7_cleanUp();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:RunningTest'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 4:
				// {{{RME port 'RQART_Timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					if( guard5_timerCompleted_event1( msg->data, (Timing::Base *)msg->sap() ) )
					{
						chain5_timerCompleted();
						return;
					}
					break;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 5:
			// {{{RME state ':TOP:Failed'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::RQARTClasses::CPP::V6_4_100_0::RQARTAbstractTestBase' port 'RQART_Exception'
				chain15_ignoreMessages();
				return;
				// }}}RME
			case 4:
				// {{{RME port 'RQART_Timer'
				chain15_ignoreMessages();
				return;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

const RTActor_class * RQARTAbstractTestHarness_Actor::getActorData( void ) const
{
	return &RQARTAbstractTestHarness_Actor::rtg_class;
}

const RTActor_class RQARTAbstractTestHarness_Actor::rtg_class =
{
	&RQARTAbstractTestBase_Actor::rtg_class
  , rtg_state_names
  , 5
  , RQARTAbstractTestHarness_Actor::rtg_parent_state
  , &RQARTAbstractTestHarness
  , 1
  , RQARTAbstractTestHarness_Actor::rtg_capsule_roles
  , 5
  , RQARTAbstractTestHarness_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 11
  , RQARTAbstractTestHarness_Actor::rtg_RQARTAbstractTestHarness_fields
};

const RTStateId RQARTAbstractTestHarness_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
  , 1
  , 1
};

const RTComponentDescriptor RQARTAbstractTestHarness_Actor::rtg_capsule_roles[] =
{
	{
		"RQART_TCPServer"
	  , &RQARTTCPServer
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, RQART_TCPServer )
	  , 1
	  , RTComponentDescriptor::Fixed
	  , 1
	  , 1 // cardinality
	  , 1
	  , rtg_interfaces_RQART_TCPServer
	  , 1
	  , rtg_bindings_RQART_TCPServer
	}
};

const RTPortDescriptor RQARTAbstractTestHarness_Actor::rtg_ports[] =
{
	{
		"RQART_Exception"
	  , (const char *)0
	  , &Exception::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, RQARTAbstractTestBase_Actor::RQART_Exception )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"RQART_Frame"
	  , (const char *)0
	  , &Frame::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, RQARTAbstractTestBase_Actor::RQART_Frame )
	  , 1 // cardinality
	  , 2
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"RQART_Log"
	  , (const char *)0
	  , &Log::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, RQARTAbstractTestBase_Actor::RQART_Log )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"RQART_Timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, RQARTAbstractTestHarness_Actor::RQART_Timer )
	  , 1 // cardinality
	  , 4
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"addIn"
	  , (const char *)0
	  , &RQARTAddInProtocol::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, RQARTAbstractTestHarness_Actor::addIn )
	  , 1 // cardinality
	  , 5
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
};

const RTFieldDescriptor RQARTAbstractTestHarness_Actor::rtg_RQARTAbstractTestHarness_fields[] =
{
	// {{{RME classAttribute 'timeoutTid'
	{
		"timeoutTid"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, timeoutTid )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_RTTimerId
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nCompletedDrivers'
  , {
		"nCompletedDrivers"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nCompletedDrivers )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nNumDrivers'
  , {
		"nNumDrivers"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nNumDrivers )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , &rtg_tm_RQARTAbstractTestHarness_Actor_nNumDrivers
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nMaxTime'
  , {
		"nMaxTime"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nMaxTime )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nCurrentTest'
  , {
		"nCurrentTest"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nCurrentTest )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nTimerCount'
  , {
		"nTimerCount"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nTimerCount )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nMinRunTimes'
  , {
		"nMinRunTimes"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nMinRunTimes )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , &rtg_tm_RQARTAbstractTestHarness_Actor_nMinRunTimes
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nMaxInstances'
  , {
		"nMaxInstances"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nMaxInstances )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nCurrentInstances'
  , {
		"nCurrentInstances"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nCurrentInstances )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nEnvironmentIndex'
  , {
		"nEnvironmentIndex"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nEnvironmentIndex )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nRQARTControllerVersion'
  , {
		"nRQARTControllerVersion"
	  , RTOffsetOf( RQARTAbstractTestHarness_Actor, nRQARTControllerVersion )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
