// {{{RME classifier 'Logical View::RQARTClasses::CPP::V6_4_100_0::RQARTAbstractTestWrapper'

#if defined( PRAGMA ) && ! defined( PRAGMA_IMPLEMENTED )
#pragma implementation "RQARTAbstractTestWrapper.h"
#endif

#include <RTSystem/VVIG.h>
#include <RQARTAbstractTestWrapper.h>
#include <RQARTExceptionCode.h>
#include <RQARTStartTestNData.h>

// {{{RME tool 'OT::Cpp' property 'ImplementationPreface'
// {{{USR

// }}}USR
// }}}RME

static const RTRelayDescriptor rtg_relays[] =
{
	{
		"RQART_TestHarness"
	  , &RQARTTestDriverProtocol::Conjugate::rt_class
	  , 1 // cardinality
	}
};

static RTActor * new_RQARTAbstractTestWrapper_Actor( RTController * _rts, RTActorRef * _ref )
{
	return new RQARTAbstractTestWrapper_Actor( _rts, _ref );
}

const RTActorClass RQARTAbstractTestWrapper =
{
	&RQARTAbstractTestBase
  , "RQARTAbstractTestWrapper"
  , (RTVersionId)0
  , 1
  , rtg_relays
  , new_RQARTAbstractTestWrapper_Actor
};

static const char * const rtg_state_names[] =
{
	"TOP"
  , "Failed"
  , "WaitingToStart"
  , "WaitingForTimer"
  , "IsMinRunTimeComplete"
};

#define SUPER RQARTAbstractTestBase_Actor

RQARTAbstractTestWrapper_Actor::RQARTAbstractTestWrapper_Actor( RTController * rtg_rts, RTActorRef * rtg_ref )
	: RQARTAbstractTestBase_Actor( rtg_rts, rtg_ref )
	, nCoregionMsgs( 0 )
	, bMinRunTimeCompleted( false )
	, nInstanceIndex( 0 )
	, nCurrentMessageIndex( 0 )
{
}

RQARTAbstractTestWrapper_Actor::~RQARTAbstractTestWrapper_Actor( void )
{
}

// {{{RME operation 'StartMinRunTimeTimer(int)'
void RQARTAbstractTestWrapper_Actor::StartMinRunTimeTimer( int nSeconds )
{
	// {{{USR
	// Set the minimum run time for the driver
	bMinRunTimeCompleted = true;
	if (nSeconds > 0)
	{
		RTTimerId minRunTimeTid = RQART_Timer.informIn(RTTimespec(nSeconds, 0));
		if (minRunTimeTid.isValid())
			bMinRunTimeCompleted = false;
		else
			RQART_Log.log("Driver timer could not be started.");
	}
	// }}}USR
}
// }}}RME

// {{{RME operation 'SendACompareFailure(const char *const)'
void RQARTAbstractTestWrapper_Actor::SendACompareFailure( const char * const pMsg )
{
	// {{{USR
	// This method takes the input string pMsg, prefixes it with
	// "$# FAIL <i> <m> #$ " where <i> is the instance index and <m> is the current
	// message index and terminates it with a carriage return and a newline.
	// The message is sent to the test harness as a CompareFailure. CompareFailure's are
	// saved with the sequence diagram generated from the trace of the test run.
	// The special prefix is used by Quality Architect's custom comparison feature.
	// NOTE: this method relies on the member variables nInstanceIndex and 
	// nCurrentMessageIndex being set to the correct values. The driver generator
	// for Quality Architect generates code to set the values in the driver state machine.
	// nInstanceIndex is not necessarily the left-to-right instance number from
	// the sequence diagram. Use the "Find In" context menu on the generated
	// test driver to see the instance index value.
	// nCurrentMessageIndex is not necessarily the top-to-bottom message number
	// from the sequence diagram. Use the "Find In" context menu on the generated
	// test driver to see the message index value.

	int nTotalLen = 0;
	int nILen;
	int nMLen;
	int nMsgLen;
	char istr[30];
	char mstr[30];

	nILen = RTFormat::_int(nInstanceIndex, istr, 30);
	nMLen = RTFormat::_int(nCurrentMessageIndex, mstr, 30);
	nMsgLen = RTMemoryUtil::strlen(pMsg);
	nTotalLen = 8 + nILen + 1 + nMLen + 4 + nMsgLen + 3;

	char *pFailMsg = new char [nTotalLen];
	if (!pFailMsg)
	{
		RQARTExceptionCode exCode = eRQARTMemoryError;
		RQART_Exception.error(RTTypedValue(&exCode)).raise();
		return;
	} 
	char *pStr = pFailMsg;
	RTMemoryUtil::strcpy(pStr, "$# FAIL ");
	pStr += 8;
	RTMemoryUtil::strcpy(pStr, istr);
	pStr += nILen;
	RTMemoryUtil::strcpy(pStr, " ");
	pStr += 1;
	RTMemoryUtil::strcpy(pStr, mstr);
	pStr += nMLen;
	RTMemoryUtil::strcpy(pStr, " #$ ");
	pStr += 4;
	RTMemoryUtil::strcpy(pStr, pMsg);
	pStr += nMsgLen;
	RTMemoryUtil::strcpy(pStr, "\r\n");

	RTString rtStr(pFailMsg);
	delete [] pFailMsg;

	RQART_TestHarness.CompareFailure(rtStr).send();
	// }}}USR
}
// }}}RME

// {{{RME operation 'LogAMessage(const char *const)'
void RQARTAbstractTestWrapper_Actor::LogAMessage( const char * const pMsg )
{
	// {{{USR
	// This method takes the input string pMsg and prefixes it with
	// "$# MSG <i> <m> #$ " where <i> is the instance index and <m> is the current
	// message index.
	// The message is sent to the test harness as a LogMessage. LogMessage's are
	// sent to the Rose Realtime log..
	// NOTE: this method relies on the member variables nInstanceIndex and 
	// nCurrentMessageIndex being set to the correct values. The driver generator
	// for Quality Architect generates code to set the values in the driver state machine.
	// nInstanceIndex is not necessarily the left-to-right instance number from
	// the sequence diagram. Use the "Find In" context menu on the generated
	// test driver to see the instance index value.
	// nCurrentMessageIndex is not necessarily the top-to-bottom message number
	// from the sequence diagram. Use the "Find In" context menu on the generated
	// test driver to see the message index value.

	int nTotalLen = 0;
	int nILen;
	int nMLen;
	int nMsgLen;
	char istr[30];
	char mstr[30];

	nILen = RTFormat::_int(nInstanceIndex, istr, 30);
	nMLen = RTFormat::_int(nCurrentMessageIndex, mstr, 30);
	nMsgLen = RTMemoryUtil::strlen(pMsg);
	nTotalLen = 7 + nILen + 1 + nMLen + 4 + nMsgLen + 1;

	char *pFailMsg = new char [nTotalLen];
	if (!pFailMsg)
	{
		RQARTExceptionCode exCode = eRQARTMemoryError;
		RQART_Exception.error(RTTypedValue(&exCode)).raise();
		return;
	}
	char *pStr = pFailMsg;
	RTMemoryUtil::strcpy(pStr, "$# MSG ");
	pStr += 7;
	RTMemoryUtil::strcpy(pStr, istr);
	pStr += nILen;
	RTMemoryUtil::strcpy(pStr, " ");
	pStr += 1;
	RTMemoryUtil::strcpy(pStr, mstr);
	pStr += nMLen;
	RTMemoryUtil::strcpy(pStr, " #$ ");
	pStr += 4;
	RTMemoryUtil::strcpy(pStr, pMsg);

	RTString rtStr(pFailMsg);
	delete [] pFailMsg;

	RQART_TestHarness.LogMessage(rtStr).send();
	// }}}USR
}
// }}}RME

int RQARTAbstractTestWrapper_Actor::_followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex )
{
	switch( rtg_portId )
	{
	case 0:
		// RQART_TestHarness
		if( rtg_repIndex < 1 )
		{
			rtg_end.port = &RQART_TestHarness;
			rtg_end.index = rtg_repIndex;
			return 1;
		}
		break;
	default:
		break;
	}
	return RQARTAbstractTestBase_Actor::_followInV( rtg_end, rtg_portId, rtg_repIndex );
}

// {{{RME transition ':TOP:Junction1:errorException'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition1_errorException
#define CALLSUPER RQARTAbstractTestBase_Actor::transition1_errorException( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestWrapper_Actor::transition1_errorException( const void * rtdata, Exception::Base * rtport )
{
	// {{{USR
	RQARTExceptionCode exCode = *(RQARTExceptionCode *)rtdata;

	// notify the test harness
	RQART_TestHarness.DriverFailed(exCode).send(Panic);
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Junction2:minRunTimeCompleted'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition3_minRunTimeCompleted
#define CALLSUPER RQARTAbstractTestBase_Actor::transition3_minRunTimeCompleted( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestWrapper_Actor::transition3_minRunTimeCompleted( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	bMinRunTimeCompleted = true;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:WaitingForTimer:Junction3:minRunTimeCompleted'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition5_minRunTimeCompleted
#define CALLSUPER RQARTAbstractTestBase_Actor::transition5_minRunTimeCompleted( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestWrapper_Actor::transition5_minRunTimeCompleted( const void * rtdata, Timing::Base * rtport )
{
	// {{{USR
	RQART_TestHarness.DriverCompleted().send();
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:IsMinRunTimeComplete:True'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition6_True
#define CALLSUPER RQARTAbstractTestBase_Actor::transition6_True( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestWrapper_Actor::transition6_True( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	RQART_TestHarness.DriverCompleted().send();
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

// {{{RME transition ':TOP:Junction4:ignoreStartTestN'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::transition7_ignoreStartTestN
#define CALLSUPER RQARTAbstractTestBase_Actor::transition7_ignoreStartTestN( rtdata, rtport )
INLINE_METHODS void RQARTAbstractTestWrapper_Actor::transition7_ignoreStartTestN( const RQARTStartTestNData * rtdata, RQARTTestDriverProtocol::Conjugate * rtport )
{
	// {{{USR
	// Ignore this message
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

INLINE_CHAINS void RQARTAbstractTestWrapper_Actor::chain2_Initial( void )
{
	// transition ':TOP:Initial:Initial'
	rtgChainBegin( 1, "Initial" );
	rtgTransitionBegin();
	rtgTransitionEnd();
	enterState( 3 );
}

INLINE_CHAINS void RQARTAbstractTestWrapper_Actor::chain1_errorException( void )
{
	// transition ':TOP:Junction1:errorException'
	rtgChainBegin( 1, "errorException" );
	exitToChainState( 1, rtg_parent_state );
	rtgTransitionBegin();
	transition1_errorException( msg->data, (Exception::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 2 );
}

INLINE_CHAINS void RQARTAbstractTestWrapper_Actor::chain7_ignoreStartTestN( void )
{
	// transition ':TOP:Junction4:ignoreStartTestN'
	rtgChainBegin( 1, "ignoreStartTestN" );
	rtgTransitionBegin();
	transition7_ignoreStartTestN( (const RQARTStartTestNData *)msg->data, (RQARTTestDriverProtocol::Conjugate *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTAbstractTestWrapper_Actor::chain3_minRunTimeCompleted( void )
{
	// transition ':TOP:Junction2:minRunTimeCompleted'
	rtgChainBegin( 1, "minRunTimeCompleted" );
	rtgTransitionBegin();
	transition3_minRunTimeCompleted( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTAbstractTestWrapper_Actor::chain8_ignoreMessages( void )
{
	// transition ':TOP:Failed:Junction2:ignoreMessages'
	rtgChainBegin( 2, "ignoreMessages" );
	rtgTransitionBegin();
	rtgTransitionEnd();
}

INLINE_CHAINS void RQARTAbstractTestWrapper_Actor::chain5_minRunTimeCompleted( void )
{
	// transition ':TOP:WaitingForTimer:Junction3:minRunTimeCompleted'
	rtgChainBegin( 4, "minRunTimeCompleted" );
	exitState( rtg_parent_state );
	rtgTransitionBegin();
	transition5_minRunTimeCompleted( msg->data, (Timing::Base *)msg->sap() );
	rtgTransitionEnd();
	enterState( 3 );
}

void RQARTAbstractTestWrapper_Actor::rtsBehavior( int signalIndex, int portIndex )
{
	for( int stateIndex = getCurrentState(); ; stateIndex = rtg_parent_state[ stateIndex - 1 ] )
		switch( stateIndex )
		{
		case 1:
			// {{{RME state ':TOP'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					chain2_Initial();
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::RQARTClasses::CPP::V6_4_100_0::RQARTAbstractTestBase' port 'RQART_Exception'
				switch( signalIndex )
				{
				case Exception::Base::rti_error:
					chain1_errorException();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 4:
				// {{{RME port 'RQART_TestHarness'
				switch( signalIndex )
				{
				case RQARTTestDriverProtocol::Conjugate::rti_StartTestN:
					chain7_ignoreStartTestN();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			case 5:
				// {{{RME port 'RQART_Timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain3_minRunTimeCompleted();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			unexpectedMessage();
			return;
			// }}}RME
		case 2:
			// {{{RME state ':TOP:Failed'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 1:
				// {{{RME classifier 'Logical View::RQARTClasses::CPP::V6_4_100_0::RQARTAbstractTestBase' port 'RQART_Exception'
				chain8_ignoreMessages();
				return;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		case 3:
			// {{{RME state ':TOP:WaitingToStart'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			default:
				break;
			}
			break;
			// }}}RME
		case 4:
			// {{{RME state ':TOP:WaitingForTimer'
			switch( portIndex )
			{
			case 0:
				switch( signalIndex )
				{
				case 1:
					return;
				default:
					break;
				}
				break;
			case 5:
				// {{{RME port 'RQART_Timer'
				switch( signalIndex )
				{
				case Timing::Base::rti_timeout:
					chain5_minRunTimeCompleted();
					return;
				default:
					break;
				}
				break;
				// }}}RME
			default:
				break;
			}
			break;
			// }}}RME
		default:
			unexpectedState();
			return;
		}
}

// {{{RME choicePoint ':TOP:IsMinRunTimeComplete'
#define SUPERMETHOD RQARTAbstractTestBase_Actor::choicePoint1_IsMinRunTimeComplete
#define CALLSUPER RQARTAbstractTestBase_Actor::choicePoint1_IsMinRunTimeComplete( rtdata, rtport )
INLINE_METHODS int RQARTAbstractTestWrapper_Actor::choicePoint1_IsMinRunTimeComplete( const void * rtdata, RTProtocol * rtport )
{
	// {{{USR
	return bMinRunTimeCompleted;
	// }}}USR
}
#undef SUPERMETHOD
#undef CALLSUPER
// }}}RME

const RTActor_class * RQARTAbstractTestWrapper_Actor::getActorData( void ) const
{
	return &RQARTAbstractTestWrapper_Actor::rtg_class;
}

const RTActor_class RQARTAbstractTestWrapper_Actor::rtg_class =
{
	&RQARTAbstractTestBase_Actor::rtg_class
  , rtg_state_names
  , 4
  , RQARTAbstractTestWrapper_Actor::rtg_parent_state
  , &RQARTAbstractTestWrapper
  , 0
  , (const RTComponentDescriptor *)0
  , 5
  , RQARTAbstractTestWrapper_Actor::rtg_ports
  , 0
  , (const RTLocalBindingDescriptor *)0
  , 4
  , RQARTAbstractTestWrapper_Actor::rtg_RQARTAbstractTestWrapper_fields
};

const RTStateId RQARTAbstractTestWrapper_Actor::rtg_parent_state[] =
{
	0
  , 1
  , 1
  , 1
};

const RTPortDescriptor RQARTAbstractTestWrapper_Actor::rtg_ports[] =
{
	{
		"RQART_Exception"
	  , (const char *)0
	  , &Exception::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestWrapper_Actor, RQARTAbstractTestBase_Actor::RQART_Exception )
	  , 1 // cardinality
	  , 1
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"RQART_Frame"
	  , (const char *)0
	  , &Frame::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestWrapper_Actor, RQARTAbstractTestBase_Actor::RQART_Frame )
	  , 0 // excluded
	  , 2
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"RQART_Log"
	  , (const char *)0
	  , &Log::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestWrapper_Actor, RQARTAbstractTestBase_Actor::RQART_Log )
	  , 1 // cardinality
	  , 3
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
  , {
		"RQART_TestHarness"
	  , (const char *)0
	  , &RQARTTestDriverProtocol::Conjugate::rt_class
	  , RTOffsetOf( RQARTAbstractTestWrapper_Actor, RQARTAbstractTestWrapper_Actor::RQART_TestHarness )
	  , 1 // cardinality
	  , 4
	  , RTPortDescriptor::KindWired + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityPublic
	}
  , {
		"RQART_Timer"
	  , (const char *)0
	  , &Timing::Base::rt_class
	  , RTOffsetOf( RQARTAbstractTestWrapper_Actor, RQARTAbstractTestWrapper_Actor::RQART_Timer )
	  , 1 // cardinality
	  , 5
	  , RTPortDescriptor::KindSpecial + RTPortDescriptor::NotificationDisabled + RTPortDescriptor::RegisterNotPermitted + RTPortDescriptor::VisibilityProtected
	}
};

const RTFieldDescriptor RQARTAbstractTestWrapper_Actor::rtg_RQARTAbstractTestWrapper_fields[] =
{
	// {{{RME classAttribute 'nCoregionMsgs'
	{
		"nCoregionMsgs"
	  , RTOffsetOf( RQARTAbstractTestWrapper_Actor, nCoregionMsgs )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'bMinRunTimeCompleted'
  , {
		"bMinRunTimeCompleted"
	  , RTOffsetOf( RQARTAbstractTestWrapper_Actor, bMinRunTimeCompleted )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_bool
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nInstanceIndex'
  , {
		"nInstanceIndex"
	  , RTOffsetOf( RQARTAbstractTestWrapper_Actor, nInstanceIndex )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
	// {{{RME classAttribute 'nCurrentMessageIndex'
  , {
		"nCurrentMessageIndex"
	  , RTOffsetOf( RQARTAbstractTestWrapper_Actor, nCurrentMessageIndex )
		// {{{RME tool 'OT::CppTargetRTS' property 'TypeDescriptor'
	  , &RTType_int
		// }}}RME
		// {{{RME tool 'OT::CppTargetRTS' property 'GenerateTypeModifier'
	  , (const RTTypeModifier *)0
		// }}}RME
	}
	// }}}RME
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'ImplementationEnding'
// {{{USR

// }}}USR
// }}}RME

// }}}RME
