// {{{RME classifier 'Logical View::RQARTClasses::CPP::V6_4_100_0::RQARTTCPServer'

#ifndef RQARTTCPServer_H
#define RQARTTCPServer_H

#ifdef PRAGMA
#pragma interface "RQARTTCPServer.h"
#endif

#include <RTSystem/VVIG.h>
#include <RQARTAddInProtocol.h>

// {{{RME tool 'OT::Cpp' property 'HeaderPreface'
// {{{USR
#include <RTTcpSocket.h>

// Define in messages
#define TEST_COMPLETED 0
#define TEST_TIMED_OUT 1
#define TEST_FAILED 2
#define TEST_CLEANED_UP 3
#define TS_LOG_MESSAGE	4
#define TS_COMPARE_FAILURE 5
#define RESET_COMPLETED 6

// Define out messages
#define TEST_ENDIAN 0
#define RESET 1
#define SET_MAX_TIME 2
#define CLEANUP_TEST 3
#define START_TEST 4
#define SET_RQART_CONTROLLER_VERSION 5

#define TS_MESSAGE_SIZE(nSize) 1 + nSize * sizeof(unsigned long)
#define TS_MAXNUMS 255
#define TS_MAXSIZE TS_MESSAGE_SIZE(TS_MAXNUMS)
// }}}USR
// }}}RME

extern const RTActorClass RQARTTCPServer;

#define SUPER RTActor
class RQARTTCPServer_Actor : public RTActor
{
public:
	// {{{RME tool 'OT::Cpp' property 'PublicDeclarations'
	// {{{USR

	// }}}USR
	// }}}RME

protected:
	// {{{RME tool 'OT::Cpp' property 'ProtectedDeclarations'
	// {{{USR

	// }}}USR
	// }}}RME

private:
	// {{{RME tool 'OT::Cpp' property 'PrivateDeclarations'
	// {{{USR

	// }}}USR
	// }}}RME

protected:
	// {{{RME classAttribute 'c_socket'
	RTTcpSocket c_socket;
	// }}}RME
	// {{{RME classAttribute 'anBuffer'
	int anBuffer[ TS_MAXNUMS ];
	// }}}RME
	// {{{RME classAttribute 'nPort'
	int nPort;
	// }}}RME
	// {{{RME classAttribute 'nSizeRemaining'
	int nSizeRemaining;
	// }}}RME
	// {{{RME classAttribute 'nBodyRemaining'
	int nBodyRemaining;
	// }}}RME
	// {{{RME classAttribute 'readBuffer'
	char readBuffer[ TS_MAXSIZE ];
	// }}}RME
	// {{{RME classAttribute 'nNums'
	unsigned long nNums;
	// }}}RME
	// {{{RME classAttribute 'nSize'
	unsigned long nSize;
	// }}}RME
	// {{{RME classAttribute 'listener'
	RTTcpSocket listener;
	// }}}RME
	// {{{RME port 'AddIn'
	RQARTAddInProtocol::Conjugate AddIn;
	// }}}RME
	// {{{RME port 'RQART_Timer'
	Timing::Base RQART_Timer;
	// }}}RME
	// {{{RME port 'RQART_Log'
	Log::Base RQART_Log;
	// }}}RME

public:
	RQARTTCPServer_Actor( RTController * rtg_rts, RTActorRef * rtg_ref );
	virtual ~RQARTTCPServer_Actor( void );

protected:
	// {{{RME operation 'processIPCData()'
	inline void processIPCData( void )
	{
		// {{{USR
		int nMessage;
		int *pnData;

		if(!ReceiveMessage()) 
		{
			return;
		}

		nMessage = anBuffer[0];
		pnData = anBuffer + 1;

		switch(nMessage)
		{
			case TEST_ENDIAN:
				// The value sent indicates the harness version
				SendInt(1, 2);
				break;
			case START_TEST: 
				AddIn.StartTest(pnData).send();
				break; 
			case RESET:
				AddIn.Reset().send();
				break;
			case SET_MAX_TIME:
				AddIn.SetMaxTime(*pnData).send();
				break;
			case SET_RQART_CONTROLLER_VERSION:
				AddIn.SetRQARTControllerVersion(*pnData).send();
				break;
			case CLEANUP_TEST:
				AddIn.CleanUpTest().send();
				break;
		}
		// }}}USR
	}
	// }}}RME
	// {{{RME operation 'SendInt(char,int)'
	void SendInt( char cMessage, int nData );
	// }}}RME
	// {{{RME operation 'ReceiveMessage()'
	int ReceiveMessage( void );
	// }}}RME
	// {{{RME operation 'SendChars(char,char *,int)'
	void SendChars( char cMessage, char * pcData, int nSize );
	// }}}RME
	// {{{RME operation 'IPCReturn(int,bool,bool)'
	int IPCReturn( int nReturn, bool bReset, bool bSetTimer = true );
	// }}}RME

public:
	virtual int _followInV( RTBindingEnd & rtg_end, int rtg_portId, int rtg_repIndex );

protected:
	// {{{RME transition ':TOP:WaitForPort:Junction2:SetPort'
	INLINE_METHODS void transition3_SetPort( const int * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:HaveListener:True'
	INLINE_METHODS void transition4_True( const int * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:HaveConnection:True'
	INLINE_METHODS void transition6_True( const void * rtdata, Timing::Base * rtport );
	// }}}RME
	// {{{RME transition ':TOP:HaveConnection:False'
	INLINE_METHODS void transition7_False( const void * rtdata, Timing::Base * rtport );
	// }}}RME
	// {{{RME transition ':TOP:Operational:Junction1:TestCompleted'
	INLINE_METHODS void transition8_TestCompleted( const void * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:Operational:Junction4:TestTimedOut'
	INLINE_METHODS void transition9_TestTimedOut( const void * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:Operational:Junction6:TestCleanedUp'
	INLINE_METHODS void transition10_TestCleanedUp( const void * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:Operational:Junction3:LogMessage'
	INLINE_METHODS void transition11_LogMessage( const RTString * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:Operational:Junction12:TestFailed'
	INLINE_METHODS void transition12_TestFailed( const int * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:Operational:Junction14:CompareFailure'
	INLINE_METHODS void transition13_CompareFailure( const RTString * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME
	// {{{RME transition ':TOP:Operational:Junction8:timeout'
	INLINE_METHODS void transition14_timeout( const void * rtdata, Timing::Base * rtport );
	// }}}RME
	// {{{RME transition ':TOP:Operational:Junction16:ResetCompleted'
	INLINE_METHODS void transition15_ResetCompleted( const void * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME

private:
	INLINE_CHAINS void chain2_Initial( void );
	INLINE_CHAINS void chain8_TestCompleted( void );
	INLINE_CHAINS void chain9_TestTimedOut( void );
	INLINE_CHAINS void chain12_TestFailed( void );
	INLINE_CHAINS void chain10_TestCleanedUp( void );
	INLINE_CHAINS void chain11_LogMessage( void );
	INLINE_CHAINS void chain13_CompareFailure( void );
	INLINE_CHAINS void chain15_ResetCompleted( void );
	INLINE_CHAINS void chain14_timeout( void );
	INLINE_CHAINS void chain3_SetPort( void );

protected:
	// {{{RME choicePoint ':TOP:HaveListener'
	INLINE_METHODS int choicePoint1_HaveListener( const int * rtdata, RQARTAddInProtocol::Conjugate * rtport );
	// }}}RME

private:
	INLINE_CHAINS void chain4_True( void );
	INLINE_CHAINS void chain1_False( void );
	INLINE_CHAINS void chain5_timeout( void );

protected:
	// {{{RME choicePoint ':TOP:HaveConnection'
	INLINE_METHODS int choicePoint2_HaveConnection( const void * rtdata, Timing::Base * rtport );
	// }}}RME

private:
	INLINE_CHAINS void chain6_True( void );
	INLINE_CHAINS void chain7_False( void );

public:
	virtual void rtsBehavior( int signalIndex, int portIndex );
	virtual const RTActor_class * getActorData( void ) const;

protected:
	static const RTActor_class rtg_class;

public:
	static const RTStateId rtg_parent_state[];

private:
	static const RTPortDescriptor rtg_ports[];

public:
	static const RTFieldDescriptor rtg_RQARTTCPServer_fields[];
};
#undef SUPER

// {{{RME tool 'OT::Cpp' property 'HeaderEnding'
// {{{USR

// }}}USR
// }}}RME

#endif /* RQARTTCPServer_H */

// }}}RME
